/*
 * This file goes with the cDIRT C++ image registration package.
 * 
 * It implements some of the methods described in the following paper :
 * Groupwise Geometric and Photometric Direct Image Registration
 * A. Bartoli in IEEE Transactions on Pattern Analysis and Machine Intelligence,
 * Vol. 30, No. 12, pp. 2098-2108, December 2008.
 * Should you use cDIRT, please cite the above paper.
 *
 * cDIRT has been implemented by Pierre Georgel and packaged by Ludovic Magerand.
 * It is based on the (more comprehensive) Matlab implementation by Adrien Bartoli.
 *
 */


#include "cdirt.h"

#include <iostream>
#include <cstdio>
#include <fstream>
#include <string>
#ifdef _DEBUG
#endif

#include <time.h>

void CDIRT_getTargetROI(CDIRT_struct *data, CvMat* tgtPoints) {
	
	if (data->converged) {
		float srcPoints[12];
		CvMat _srcPoints = cvMat(3,4,CV_32F, srcPoints);
		srcPoints[0] = srcPoints[3] = static_cast<float>(data->ROI.x+ 1 );
		srcPoints[1] = srcPoints[2] = static_cast<float>(data->ROI.x + 1 + data->ROI.width);
		srcPoints[4] = srcPoints[5] = static_cast<float>(data->ROI.y+ 1 );
		srcPoints[6] = srcPoints[7] = static_cast<float>(data->ROI.y + 1 + data->ROI.height);
		srcPoints[8] = srcPoints[9] = srcPoints[10] = srcPoints[11] = 1.0;

		cvMatMul(data->G, &_srcPoints, tgtPoints); 
		//normalized!!!
		for (int idxCol=0; idxCol < 4; ++idxCol) {
			CV_MAT_ELEM(*tgtPoints, float,0,idxCol) 
				= CV_MAT_ELEM(*tgtPoints, float,0,idxCol) / CV_MAT_ELEM(*tgtPoints, float,2,idxCol) - 1.0f;
			CV_MAT_ELEM(*tgtPoints, float,1,idxCol) 
				= CV_MAT_ELEM(*tgtPoints, float,1,idxCol) / CV_MAT_ELEM(*tgtPoints, float,2,idxCol) - 1.0f;
		}
	}
}


bool badWarp(const CvMat *H, int w, int h, CvRect ROI) {
		CvMat* p = cvCreateMat(3,1, CV_32F);
		CvMat* p_ = cvCreateMat(3,1, CV_32F);

	CV_MAT_ELEM(*p, float,0,0) = static_cast<float>(ROI.x + 1);
	CV_MAT_ELEM(*p, float,1,0) = static_cast<float>(ROI.y + 1);
	CV_MAT_ELEM(*p, float,2,0) = 1.0;
	cvMatMul(H,p,p_); cvScale(p_,p_, 1.0 / CV_MAT_ELEM(*p_,float,2,0));
	
	bool res = true;

	if (!(CV_MAT_ELEM(*p_,float,0,0) < 1 || CV_MAT_ELEM(*p_,float,0,0) >= w  -1
		|| CV_MAT_ELEM(*p_,float,1,0) < 1 || CV_MAT_ELEM(*p_,float,1,0) >= h -1)) {
		CV_MAT_ELEM(*p, float,0,0) = static_cast<float>(ROI.x + 1);
		CV_MAT_ELEM(*p, float,1,0) = static_cast<float>(ROI.y + ROI.height);
		CV_MAT_ELEM(*p, float,2,0) = 1.0;
		cvMatMul(H,p,p_); cvScale(p_,p_, 1.0 / CV_MAT_ELEM(*p_,float,2,0));
		
		if (!(CV_MAT_ELEM(*p_,float,0,0) < 1 || CV_MAT_ELEM(*p_,float,0,0) >= w  -1
		|| CV_MAT_ELEM(*p_,float,1,0) < 1 || CV_MAT_ELEM(*p_,float,1,0) >= h -1)) {
			CV_MAT_ELEM(*p, float,0,0) = static_cast<float>(ROI.x + ROI.width);
			CV_MAT_ELEM(*p, float,1,0) = static_cast<float>(ROI.y + 1);
			CV_MAT_ELEM(*p, float,2,0) = 1.0;
			cvMatMul(H,p,p_); cvScale(p_,p_, 1.0 / CV_MAT_ELEM(*p_,float,2,0));
			
			if (!(CV_MAT_ELEM(*p_,float,0,0) < 1 || CV_MAT_ELEM(*p_,float,0,0) >= w -1 
				|| CV_MAT_ELEM(*p_,float,1,0) < 1 || CV_MAT_ELEM(*p_,float,1,0) >= h -1)) {
				
					CV_MAT_ELEM(*p, float,0,0) = static_cast<float>(ROI.x + ROI.width);
					CV_MAT_ELEM(*p, float,1,0) = static_cast<float>(ROI.y + ROI.height);
					CV_MAT_ELEM(*p, float,2,0) = 1.0;
					cvMatMul(H,p,p_); cvScale(p_,p_, 1.0 / CV_MAT_ELEM(*p_,float,2,0));
					
					if (!(CV_MAT_ELEM(*p_,float,0,0) < 1 || CV_MAT_ELEM(*p_,float,0,0) >= w -1
						|| CV_MAT_ELEM(*p_,float,1,0) < 1 || CV_MAT_ELEM(*p_,float,1,0) >= h  -1)) {
						res = false;
					}
			}
		}
	}
	cvReleaseMat(&p);
	cvReleaseMat(&p_);

	return res;
}

int warpImage(const IplImage *src, IplImage *dst, CvMat *H, CvRect ROI)
	{
		
		if (src->depth != IPL_DEPTH_32F || dst->depth != IPL_DEPTH_32F || 
			dst->nChannels !=  src->nChannels || dst->width != ROI.width || dst->height != ROI.height) {
			printf("in warpImage, Incongruence !! \n");
			return -1;
		}
		
		if (badWarp(H, src->width,src->height, ROI)) {
			printf("in warpImage, badWarp\n");
			return -1;
		}
		
		for (int y = 0; y < dst->height; ++y)
		{
			for (int x = 0; x < dst->width; ++x)
			{
				
				float xROI = static_cast<float>(x + ROI.x + 1);
					float yROI = static_cast<float>(y + ROI.y + 1);

					float x_ = CV_MAT_ELEM(*H,float,0,0)*xROI+CV_MAT_ELEM(*H,float,0,1)*yROI+CV_MAT_ELEM(*H,float,0,2);
					float y_ = CV_MAT_ELEM(*H,float,1,0)*xROI+CV_MAT_ELEM(*H,float,1,1)*yROI+CV_MAT_ELEM(*H,float,1,2);
					float w_ = 1.0f / (CV_MAT_ELEM(*H,float,2,0)*xROI+CV_MAT_ELEM(*H,float,2,1)*yROI+CV_MAT_ELEM(*H,float,2,2));

					x_ = x_ * w_ -1.0f;
					y_ = y_ * w_ -1.0f;


				int x_f = static_cast<int>(x_);
					int y_f = static_cast<int>(y_);
					float dx = x_-floor(x_);
					float dy = y_-floor(y_);
					float k00 = (1.0f-dx)*(1.0f-dy);
					float k10 = (dx)*(1.0f-dy);
					float k01 = (1.0f-dx)*(dy);
					float k11 = (dx)*(dy); 
					for (int c=0; c < dst->nChannels; ++c) {
						CV_IMAGE_ELEM(dst,float,y,dst->nChannels * x + c) = 
							k00 *CV_IMAGE_ELEM(src,float,y_f,dst->nChannels *x_f+c)+
							k10 *CV_IMAGE_ELEM(src,float,y_f,dst->nChannels *(x_f+1)+c)+
							k01 *CV_IMAGE_ELEM(src,float,y_f+1,dst->nChannels *x_f+c)+
							k11 *CV_IMAGE_ELEM(src,float,(y_f+1),dst->nChannels *(x_f+1)+c);
						 
					}
			}
		
		}

		return 0;
	}


int writeCvMatToMfile(const CvMat* src, const char *variableName) {
	using namespace std;
	if (!variableName) {
		cerr << "(@writeCvMatToMfile): No variable name defined, exit function" << endl;
		exit(-1);
	}
	
	string fileNameS(variableName);
	fileNameS += "_script.m";
	
	
	// if file exist remove it.
	remove( fileNameS.c_str());
	
	ofstream myfile;
	
	myfile.open(fileNameS.c_str()); 
	if (myfile.fail()) { 
		cout << "Cannot open " << fileNameS << endl; 
		return -1; 
   } 
	
	//write variable name
	myfile << variableName << " = [ ..." << endl;
	
	for (int rowIdx = 0; rowIdx < src->rows; rowIdx++) {
		for (int colIdx = 0; colIdx < src->cols; colIdx++)
			myfile << src->data.fl[rowIdx * src->cols + colIdx] << "   ";
	myfile << endl;  
	}

	myfile << "];" <<endl;
	myfile.close();


	return 0;
}
int saveCdirtStructure(const std::string & filename, CDIRT_struct * data) 
{
	using namespace std;

	string ext(".m");
	string filename_ext( filename + ext);

	ofstream filep(filename_ext.c_str());
	if (filep.is_open()) {
		filep << "S = struct;" << endl;
		filep << "S.initErr = " << data->initErr << ";" << endl;
		filep << "S.finalErr = " << data->finalErr << ";" << endl;
	filep << "S.G" << " = [ ..." << endl;
	
	for (int rowIdx = 0; rowIdx < 3; rowIdx++) {
		for (int colIdx = 0; colIdx < 3; colIdx++)
			filep << data->G->data.fl[rowIdx * 3 + colIdx] << "   ";
	filep << endl;  
	}

	filep << "];" <<endl;
	filep << "S.P" << " = [ " << data->P->data.fl[0] << " " <<  data->P->data.fl[1] << "];"  << endl ;
	filep.close();
	}
	
	return 0;
}

float findMaxVal(const IplImage* src) 
{
	float maxVal = 0.0;
	src->nChannels;
	for (int yIdx =0; yIdx <src->height; ++yIdx)
		for (int xIdx =0; xIdx < (src->nChannels * src->width); ++xIdx)
			maxVal = MAX(maxVal, CV_IMAGE_ELEM(src, float, yIdx, xIdx));
	
	return maxVal;
}


void CDIRT_Precomputation(const IplImage* src, CDIRT_struct* data, CvRect ROI)
{
	data->ROI = ROI;	

	int w = data->w =  ROI.width;
	int h = data->h = ROI.height;
	data->nChannels = src->nChannels;
	
	int wh = w*h;
	int nparam = 10; //num DoF

	data->sI_ROI = cvCreateMat(wh*data->nChannels, 1, CV_32FC1);
	
	IplImage* sI_ROIT = cvCreateImage(cvSize(data->w,data->h), IPL_DEPTH_32F, data->nChannels);//cleared
	
	IplImage* dummy_sI_ROI = cvCreateImageHeader(cvSize(data->h,data->w), IPL_DEPTH_32F, data->nChannels); //cleared
	cvSetData(dummy_sI_ROI,data->sI_ROI->data.fl,CV_AUTOSTEP);

	IplImage* sI = cvCreateImage(cvSize(src->width,src->height), IPL_DEPTH_32F, data->nChannels); //cleared
	IplImage* filteredI = cvCreateImage(cvSize(src->width,src->height), IPL_DEPTH_32F, data->nChannels); //cleared
	

	CvMat *sIrT = cvCreateMat(wh*data->nChannels, 1, CV_32FC1); //cleared
	IplImage* dummy_sIrT = cvCreateImageHeader(cvSize(w,h), IPL_DEPTH_32F, data->nChannels); //cleared
	cvSetData(dummy_sIrT,sIrT->data.fl,CV_AUTOSTEP);

	CvMat *sIr = cvCreateMat(wh*data->nChannels, 1, CV_32FC1); //cleared
	IplImage* dummy_sIr = cvCreateImageHeader(cvSize(h,w), IPL_DEPTH_32F, data->nChannels); //cleared
	cvSetData(dummy_sIr,sIr->data.fl,CV_AUTOSTEP);


	CvMat *sIcT = cvCreateMat(wh*data->nChannels, 1, CV_32FC1); //cleared
	IplImage* dummy_sIcT = cvCreateImageHeader(cvSize(w,h), IPL_DEPTH_32F, data->nChannels); //cleared
	cvSetData(dummy_sIcT,sIcT->data.fl,CV_AUTOSTEP);

	CvMat *sIc = cvCreateMat(wh*data->nChannels, 1, CV_32FC1); //cleared
	IplImage* dummy_sIc = cvCreateImageHeader(cvSize(h,w), IPL_DEPTH_32F, data->nChannels); //cleared
	cvSetData(dummy_sIc,sIc->data.fl,CV_AUTOSTEP);

	data->SD_ROI = cvCreateMat( nparam, wh*data->nChannels, CV_32FC1);
	
	cvConvert(src, sI);
	cvSetImageROI(sI, ROI);
	cvCopy(sI,sI_ROIT);
	cvResetImageROI(sI);

	float pnorm_params = 1.0;

	if (data->options[0] == CDIRT_PNORM) 
		pnorm_params = findMaxVal(sI_ROIT);
	//normalize the image
	data->pnorm = pnorm_params;

	
	float sigma = 1.0;
	CvMat* kernelX = cvCreateMat(1,7,CV_32F);
	
	//x .* exp(- (x.^2) / (2*sigma^2));
	CV_MAT_ELEM(*kernelX,float,0,0) = -3.0f * GAUSSIAN((-3.0f),sigma);
	CV_MAT_ELEM(*kernelX,float,0,1) = -2.0f * GAUSSIAN((-2.0f),sigma);
	CV_MAT_ELEM(*kernelX,float,0,2) = -1.0f * GAUSSIAN((-1.0f),sigma);
	CV_MAT_ELEM(*kernelX,float,0,3) = 0.0f * GAUSSIAN(0.0f,sigma);
	CV_MAT_ELEM(*kernelX,float,0,4) = 1.0f * GAUSSIAN(1.0f,sigma);
	CV_MAT_ELEM(*kernelX,float,0,5) = 2.0f * GAUSSIAN(2.0f,sigma);
	CV_MAT_ELEM(*kernelX,float,0,6) = 3.0f * GAUSSIAN(3.0f,sigma);

	float sum = 0.0;
	for (int idx=0; idx < 7; ++idx) 
		sum += abs(CV_MAT_ELEM(*kernelX,float,0,idx));
    cvScale(kernelX,kernelX,1.0 / sum);
	
	CvMat* kernelY = cvCreateMat(7,1,CV_32F);
	cvTranspose(kernelX,kernelY);

	
	cvFilter2D(sI, filteredI, kernelX);
	cvSetImageROI(filteredI, ROI);
	cvCopy(filteredI,dummy_sIcT);
	cvResetImageROI(filteredI);
	cvTranspose(dummy_sIcT,dummy_sIc);

	cvFilter2D(sI, filteredI, kernelY);
	cvSetImageROI(filteredI, ROI);
	cvCopy(filteredI,dummy_sIrT);
	cvTranspose(dummy_sIrT,dummy_sIr);

	cvResetImageROI(filteredI);
	cvTranspose(sI_ROIT,dummy_sI_ROI);


	//the image gradient must be normalized, assuming that the normalizing
 	//transformation can be applied after derivation
	cvScale(sIr, sIr, 1.0 / pnorm_params);
	cvScale(sIc, sIc, 1.0 / pnorm_params);
	cvScale(data->sI_ROI,data->sI_ROI, 1.0 / data->pnorm);


	CvMat *Gr = cvCreateMat(wh,8, CV_32F); cvSet(Gr,cvScalar(0.0)); //cleared
	CvMat *Gc = cvCreateMat(wh,8, CV_32F); cvSet(Gc,cvScalar(0.0)); //cleared

	//Gr = [ R C ones(n, 1) zeros(n, 3) -R.^2 -R.*C ];
    //Gc = [ zeros(n, 3) R C ones(n, 1) -R.*C -C.^2 ];

	for (int yIdx = 0; yIdx < w; ++yIdx) {
		for (int xIdx = 0; xIdx < h; ++xIdx) {
			float x1  = static_cast<float>(xIdx + ROI.x + 1);
			float y1  = static_cast<float>(yIdx + ROI.y + 1);
			float x1y1 = x1*y1;

			CV_MAT_ELEM(*Gr,float, xIdx + h*yIdx,0) = x1;
			CV_MAT_ELEM(*Gr,float,xIdx + h*yIdx,1) = y1;
			CV_MAT_ELEM(*Gr,float,xIdx + h*yIdx,2) = 1.0;

			CV_MAT_ELEM(*Gr,float,xIdx + h*yIdx,6) = - x1*x1;
			CV_MAT_ELEM(*Gr,float,xIdx + h*yIdx,7) = - x1y1;

			CV_MAT_ELEM(*Gc,float,xIdx + h*yIdx,3) = x1;
			CV_MAT_ELEM(*Gc,float,xIdx + h*yIdx,4) = y1;
			CV_MAT_ELEM(*Gc,float,xIdx + h*yIdx,5) = 1.0;

			CV_MAT_ELEM(*Gc,float, xIdx + h*yIdx,6) = - x1y1;
			CV_MAT_ELEM(*Gc,float, xIdx + h*yIdx,7) = - y1*y1;
			}
	}

	for (int yIdx = 0; yIdx < h; ++yIdx) {
		for (int xIdx = 0; xIdx < w; ++xIdx) {
			for (int cIdx = 0; cIdx < data->nChannels; ++cIdx) {
				for (int gIdx = 0; gIdx < 8; ++gIdx) 
					CV_MAT_ELEM(*(data->SD_ROI), float,gIdx, data->nChannels * (xIdx + w*yIdx) + cIdx) =
						CV_MAT_ELEM(*sIr, float, data->nChannels * (xIdx + w*yIdx) + cIdx,0) * CV_MAT_ELEM(*Gr,float, xIdx + w*yIdx,gIdx)
						+ 
						CV_MAT_ELEM(*sIc, float, data->nChannels * (xIdx + w*yIdx) + cIdx,0) * CV_MAT_ELEM(*Gc,float, xIdx + w*yIdx,gIdx);

				//G = [ vect(Is') ones(size(Is, 1)*3, 1) ];
				CV_MAT_ELEM(*(data->SD_ROI), float,8, data->nChannels * (xIdx + w*yIdx) + cIdx) = 
						CV_MAT_ELEM(*data->sI_ROI, float, data->nChannels * (xIdx + w*yIdx) + cIdx,0);
				CV_MAT_ELEM(*(data->SD_ROI), float,9, data->nChannels * (xIdx + w*yIdx) + cIdx) = 1.0;
			}
		}
	}

	CvMat * SDT = cvCreateMat(data->SD_ROI->cols,data->SD_ROI->rows, CV_32FC1); //cleared
	cvTranspose(data->SD_ROI,SDT);

	data->H = cvCreateMat(nparam,nparam, CV_32FC1);
	cvMatMul(data->SD_ROI,SDT,data->H);
	
	data->invH = cvCreateMat(nparam,nparam, CV_32FC1);
	cvInv(data->H,data->invH);
	
	//create of the homography that will be used to store the result.
 	data->G = cvCreateMat(3,3, CV_32FC1);
	data->P = cvCreateMat(2,1,CV_32FC1);


	data->b = cvCreateMat(10,1, CV_32FC1);
	data->dp = cvCreateMat(10,1, CV_32FC1);

	data->dH = cvCreateMat(3,3, CV_32FC1);
	data->invdH = cvCreateMat(3,3, CV_32FC1);

	data->warp = cvCreateMat(data->w*data->h*data->nChannels,1, CV_32F);
	
	data->tI = cvCreateMat(src->width * src->height * data->nChannels,1,CV_32F);
	data->tIT = cvCreateMat(src->width * src->height * data->nChannels,1,CV_32F);

	cvReleaseImageHeader(&dummy_sI_ROI);
	cvReleaseImageHeader(&dummy_sIc);
	cvReleaseImageHeader(&dummy_sIr);
	cvReleaseImageHeader(&dummy_sIcT);
	cvReleaseImageHeader(&dummy_sIrT);
	
	cvReleaseImage(&sI);
	cvReleaseImage(&filteredI);
	cvReleaseImage(&sI_ROIT);


	cvReleaseMat(&kernelX);
	cvReleaseMat(&kernelY);
	cvReleaseMat(&SDT);
	cvReleaseMat(&Gr);
	cvReleaseMat(&Gc);
	cvReleaseMat(&sIr);
	cvReleaseMat(&sIc);
	cvReleaseMat(&sIrT);
	cvReleaseMat(&sIcT);
}

void CDIRT_Registration(const IplImage *tgt,CDIRT_struct* data) 
{
#ifdef _TIMING
	long cpu_time = 0;
	long start_t = clock();
#endif

	int tgt_w = tgt->width;
	int tgt_h = tgt->height;
	int tgt_nc = tgt->nChannels;

	int tgt_wh = tgt_w * tgt_h;
	int tgt_whnc = tgt_wh * tgt_nc;
	//In debug do the dimenssion checks.
	
	CvRect ROIT = cvRect(data->ROI.y,data->ROI.x,data->ROI.height,data->ROI.width); 
	
	IplImage *warpedImage = cvCreateImageHeader(cvSize(data->h,data->w),IPL_DEPTH_32F, data->nChannels);
	cvSetData(warpedImage,data->warp->data.fl,CV_AUTOSTEP);
	

	if (data->tI->rows != tgt_whnc) {
		cvReleaseMat(&(data->tI));
		data->tI = cvCreateMat(tgt_whnc,1,CV_32F);
		cvReleaseMat(&(data->tIT));
		data->tIT = cvCreateMat(tgt_whnc,1,CV_32F);
	}

	IplImage* dummy_tI = cvCreateImageHeader(cvSize(tgt_h,tgt_w), IPL_DEPTH_32F, tgt_nc); //cleared
	cvSetData(dummy_tI,data->tI->data.fl,CV_AUTOSTEP);


	IplImage* dummy_tIT = cvCreateImageHeader(cvSize(tgt_w,tgt_h), IPL_DEPTH_32F, tgt_nc); //cleared
	cvSetData(dummy_tIT,data->tIT->data.fl,CV_AUTOSTEP);

	cvConvertScale(tgt, dummy_tIT, 1.0 / data->pnorm); //convert to float
	cvTranspose(dummy_tIT,dummy_tI);
	
	// initialize the registration //TODO change this !!! to accept initial solution [1 0 roi.x; 0 1 roi.y; 0 0 1]
	cvSetIdentity(data->G);
	
	CV_MAT_ELEM(*data->P,float,0,0) = 1;
	CV_MAT_ELEM(*data->P,float,1,0) = 0;
	

	CV_MAT_ELEM(*(data->P),float,1,0) = CV_MAT_ELEM(*(data->P),float,1,0)/ data->pnorm;
	//iteration counter
	int it = 0;

	// main loop
	bool converged = false;
	float err = 0.0;


	while (true) {
		

		//WARP image
		
		if (warpImage(dummy_tI, warpedImage, data->G, ROIT)) {
#ifdef _DEBUG
			std::cerr << "problem in warping should exist" << std::endl;
#endif 
			break;
		}

		//apply gain and bias
		cvScale(data->warp, data->warp, CV_MAT_ELEM(*(data->P), float, 0, 0));
		cvAddS(data->warp, cvScalar(CV_MAT_ELEM(*(data->P), float, 1, 0)), data->warp);
		
		//todo look at the difference here!
		//compute diff 
		cvSub(data->warp, data->sI_ROI, data->warp);
		
		//compute err
		err = 0.0;
		for (int idx = 0; idx < data->warp->rows; ++idx) {
			err+= CV_MAT_ELEM( *(data->warp),float, idx, 0) * CV_MAT_ELEM( *(data->warp),float, idx, 0);
		}
		err = sqrt(err) / sqrt(	static_cast<float>(data->warp->rows));
		
		//unormalized error
		err *= data->pnorm;
		if (!it)
			data->initErr = err;

		if (converged)
			break;
		// compute the right hand side of the normal equations
		cvMatMul(data->SD_ROI, data->warp,data->b);
		  
		// compute the update vector
		cvMatMul(data->invH, data->b, data->dp);

		 // norm of the update vector for convergence test
		float normd = static_cast<float>(cvNorm(data->dp));

		// use full inverse of the increment
        //dH = eye(3) + [ dg(1) dg(2) dg(3) ; dg(4) dg(5) dg(6) ; dg(7) dg(8) 0 ];
		
		CV_MAT_ELEM(*(data->dH),float,0,0) = 1.0f + CV_MAT_ELEM(*(data->dp),float,0,0);
		CV_MAT_ELEM(*(data->dH),float,0,1) = CV_MAT_ELEM(*(data->dp),float,1,0);
		CV_MAT_ELEM(*(data->dH),float,0,2) = CV_MAT_ELEM(*(data->dp),float,2,0);
		CV_MAT_ELEM(*(data->dH),float,1,0) = CV_MAT_ELEM(*(data->dp),float,3,0);
		CV_MAT_ELEM(*(data->dH),float,1,1) = 1.0f + CV_MAT_ELEM(*(data->dp),float,4,0);
		CV_MAT_ELEM(*(data->dH),float,1,2) = CV_MAT_ELEM(*(data->dp),float,5,0);
		CV_MAT_ELEM(*(data->dH),float,2,0) = CV_MAT_ELEM(*(data->dp),float,6,0);
        CV_MAT_ELEM(*(data->dH),float,2,1) = CV_MAT_ELEM(*(data->dp),float,7,0);
		CV_MAT_ELEM(*(data->dH),float,2,2) = 1.0f;
		//g = g *	inv(dH);
		cvInv(data->dH,data->invdH);
		cvMatMul(data->G, data->invdH, data->dH);
		float froNorm_dH = 0.0;
		for (int xIdx=0; xIdx < 3; ++xIdx)
			for (int yIdx=0; yIdx < 3; ++yIdx)
				froNorm_dH += CV_MAT_ELEM(*(data->dH), float, xIdx, yIdx)*CV_MAT_ELEM(*(data->dH), float, xIdx, yIdx);
		froNorm_dH = sqrt(froNorm_dH);
		cvScale(data->dH, data->G, 1.0 / froNorm_dH);
		




		float invda = 1 / ( 1 + CV_MAT_ELEM(*data->dp,float,8,0) );
        CV_MAT_ELEM(*data->P,float,0,0) = invda * CV_MAT_ELEM(*data->P,float,0,0);
        CV_MAT_ELEM(*data->P,float,1,0) = invda * (CV_MAT_ELEM(*data->P,float,1,0) - CV_MAT_ELEM(*data->dp,float,9,0));

		// convergence test
		if (normd < CDIRT_MIN_STEP_SIZE) {
			#ifdef _DEBUG
				std::cout << "Minimum step size reached" << std::endl;            
			#endif
			converged = true;
		}

		it++;
		#ifdef _DEBUG
			std::cout << "Error at iter " << it << " is " << err << std::endl;
		#endif
		// maximum number of iterations reached
		if (it >= CDIRT_MAX_NB_IT) {
			#ifdef _DEBUG
				std::cout << "Maximum number of iterations reachedd" << std::endl;            
			#endif
		    converged = true;
		}

	}
#ifdef _TIMING
	long stop_t = clock();
	cpu_time += stop_t - start_t;
#endif

	cvScale(data->P, data->P, data->pnorm);

	data->finalErr = err;
	data->converged = converged;
	
	cvReleaseImageHeader(&warpedImage);
	cvReleaseImageHeader(&dummy_tI);
	cvReleaseImageHeader(&dummy_tIT);
#ifdef _TIMING	
	std::cout << "Timing : " << cpu_time  << std::endl;
#endif

}

void CDIRT_releaseData(CDIRT_struct* data)
{
	cvReleaseMat(&(data->sI_ROI));
	cvReleaseMat(&(data->SD_ROI));
	cvReleaseMat(&(data->warp));
	cvReleaseMat(&(data->tI));
	cvReleaseMat(&(data->tIT));

	cvReleaseMat(&(data->b));
	cvReleaseMat(&(data->dH));
	cvReleaseMat(&(data->dp));
	cvReleaseMat(&(data->P));
	cvReleaseMat(&(data->G));
	cvReleaseMat(&(data->H));
	cvReleaseMat(&(data->invH));
	cvReleaseMat(&(data->invdH));  
}

